<?php

/*
Copyright (c) 2018, Florian Schaal, schaal @it UG
All rights reserved.
*/

//* This class contains the remote functions for the ISPConfig GDPR Plugin

class remoting_gdpr extends remoting {

	public function gdpr_contract_get($session_id, $primary_id) {
		global $app;

		if(!$this->checkPerm($session_id, 'gdpr')) {
			throw new SoapFault('permission_denied', 'You do not have the permissions to access this function.');
			return false;
		}
		
		$app->remoting_lib->loadFormDef('../gdpr/form/gdpr.tform.php');
		return $app->remoting_lib->getDataRecord($primary_id);
	}

	//* Get gdpr details for all contracts of a client
	public function gdpr_contract_get_by_client($session_id, $client_id) {
		global $app;

		$client_id = intval($client_id);
		
		$params = array('client_id' => $client_id);
		
		return $this->gdpr_contract_get($session_id, $params);
	}

	//* Get PDF file 
	public function gdpr_contract_get_file($session_id, $gdpr_contract_id) {
		global $app;

		if(!$this->checkPerm($session_id, 'gdpr')) {
			throw new SoapFault('permission_denied', 'You do not have the permissions to access this function.');
			return false;
		}

		$contract = $app->db->queryOneRecord("SELECT * FROM gdpr_contract WHERE gdpr_contract_id = ?", $gdpr_contract_id);

		$app->uses('gdpr_functions');

		$gdpr_settings = $this->get_gdpr_settings();

		$filename = $app->gdpr_functions->get_filename($contract['filename']);
        
		if($filename === false) {
			throw new SoapFault('no_such_file', 'The file file does not exists.');
			return false;
        }
		$path_parts = pathinfo($filename);
        if($gdpr_settings['db_store'] == 'y') {
            $contract_file=base64_decode($contract['contract']);
            if(!file_put_contents($filename, $contract_file)) {
				throw new SoapFault('error_write_file', 'Cannot export the contracts.');
				return false;
            }
        }

	    if(is_file($filename)) {
			$out=file_get_contents($filename);
        	if($gdpr_settings['db_store'] == 'y') @unlink($filename);
			return base64_encode($out);
    	} else {
			throw new SoapFault('error_read_file', 'Cannot read the file');
			return false;
    	}

	}

	//* Get GDPR client settings
	public function gdpr_client_settings_get($session_id, $primary_id) {

		global $app;

		if(!$this->checkPerm($session_id, 'gdpr')) {
			throw new SoapFault('permission_denied', 'You do not have the permissions to access this function.');
			return false;
		}

		$app->remoting_lib->loadFormDef('../gdpr/form/gdpr_client_settings.tform.php');
		return $app->remoting_lib->getDataRecord($primary_id);
	}

	//* Add GDPR client settings
	public function gdpr_client_settings_add($session_id, $client_id, $params) {

		if(!$this->checkPerm($session_id, 'gdpr')) {
			throw new SoapFault('permission_denied', 'You do not have the permissions to access this function.');
			return false;
		}

		return $this->insertQuery('../gdpr/form/gdpr_client_settings.tform.php', $client_id, $params);
	}

	//* Update GDPR client settings
	public function gdpr_client_settings_update($session_id, $client_id, $params) {
		if(!$this->checkPerm($session_id, 'gdpr')) {
			throw new SoapFault('permission_denied', 'You do not have the permissions to access this function.');
			return false;
		}

		$affected_rows = $this->updateQuery('../gdpr/form/gdpr_client_settings.tform.php', $client_id, $client_id, $params);
		return $affected_rows;
	}

	//* Delete Invoice client settings
	public function gdpr_client_settings_delete($session_id, $primary_id) {
		if(!$this->checkPerm($session_id, 'gdpr')) {
			throw new SoapFault('permission_denied', 'You do not have the permissions to access this function.');
			return false;
		}

		$affected_rows = $this->deleteQuery('../gdpr/form/gdpr_client_settings.tform.php', $primary_id);
		return $affected_rows;
	}

}
?>
