<?php

/*
Copyright (c) 2018, Florian Schaal, schaal @it UG
All rights reserved.
*/

class gdpr_plugin {

	var $plugin_name        = 'gdpr_plugin';
	var $class_name         = 'gdpr_plugin';

	function onLoad() {
		global $app;

		$app->plugin->registerEvent('gdpr:gdpr_company:on_check_delete', 'gdpr_plugin', 'gdpr_company_before_delete');
		$app->plugin->registerEvent('gdpr:gdpr_company:on_after_delete', 'gdpr_plugin', 'gdpr_company_delete');
		$app->plugin->registerEvent('gdpr:gdpr_company:on_after_update', 'gdpr_plugin', 'gdpr_company_update');
		$app->plugin->registerEvent('gdpr:gdpr_company:on_after_insert', 'gdpr_plugin', 'gdpr_company_update');

		$app->plugin->registerEvent('gdpr:gdpr_contract:on_after_insert', 'gdpr_plugin', 'gdpr_update');
		$app->plugin->registerEvent('gdpr:gdpr_contract:on_after_delete', 'gdpr_plugin', 'gdpr_delete');

		$app->plugin->registerEvent('gdpr:gdpr_settings:on_after_update', 'gdpr_plugin', 'gdpr_settings');
		
		$app->plugin->registerEvent('gdpr:gdpr_stored_data:on_after_insert', 'gdpr_plugin', 'gdpr_data');

		$app->plugin->registerEvent('gdpr:gdpr_admin_contract:on_after_update', 'gdpr_plugin', 'gdpr_admin_update');
		
		$app->plugin->registerEvent('client:client:on_check_delete', 'gdpr_plugin', 'client_before_delete');
		$app->plugin->registerEvent('client:client:on_after_update', 'gdpr_plugin', 'client_update');
		$app->plugin->registerEvent('client:client:on_after_insert', 'gdpr_plugin', 'client_insert');
		$app->plugin->registerEvent('client:client:on_after_delete', 'gdpr_plugin', 'client_delete');

	}

	function gdpr_settings($event_name, $page_form) {
		global $app;

		$app->uses('gdpr_functions');

		$dataRecord=$page_form->dataRecord;
		$oldDataRecord=$page_form->oldDataRecord;
		if(isset($dataRecord['db_store'])) {
			if ($dataRecord['db_store'] != $oldDataRecord['db_store']) {
				if ($dataRecord['db_store'] == 'y') $app->gdpr_functions->move_to_db();
			}
		} else {
			if($oldDataRecord['db_store'] == 'y') $app->gdpr_functions->move_to_file();
		}
	}
	
	function gdpr_company_before_delete($event_name, $page_form) {
		global $app;

		include ISPC_WEB_PATH.'/gdpr/lib/lang/'.$_SESSION['s']['language'].'.lng';

		$dataRecord=$page_form->dataRecord;
		if($_SESSION['s']['user']['typ'] != 'admin') $app->error($app->lng('error_no_delete_permission'));

		$in_use = false;

		$temp=$app->db->queryAllRecords('SELECT gdpr_company_id FROM gdpr_client_settings WHERE gdpr_company_id = ?', $dataRecord['gdpr_company_id']);
        if(is_array($temp) && !empty($temp)) $in_use = true;

        if(!$in_use){
			$temp=$app->db->queryAllRecords('SELECT company_id FROM gdpr_contract WHERE company_id = ?', $dataRecord['gdpr_company_id']);
        	if(is_array($temp) && !empty($temp)) $in_use = true;
        }
        if(!$in_use){
			$temp=$app->db->queryAllRecords('SELECT company_id FROM gdpr_templates WHERE company_id = ?', $dataRecord['gdpr_company_id']);
        	if(is_array($temp) && !empty($temp)) $in_use = true;
        }
        if(!$in_use){
			$temp=$app->db->queryAllRecords('SELECT company_id FROM gdpr_email_templates WHERE company_id = ?', $dataRecord['gdpr_company_id']);
        	if(is_array($temp) && !empty($temp)) $in_use = true;
        }

		if($in_use) $app->error($wb['gdpr_company_has_data_txt']);

	}

	function gdpr_company_delete($event_name, $page_form) {
		global $app;

		$app->db->query('DELETE FROM gdpr_templates, gdpr_email_templates WHERE company_id = ?', $page_form->dataRecord['gdpr_company_id']);
	}

	function gdpr_company_update($event_name, $page_form) {
		global $app;

		$dataRecord=$page_form->dataRecord;
		$oldDataRecord=$page_form->oldDataRecord;
		if($dataRecord['logo_img'] == '-1') {
			$temp=$app->db->queryOneRecord("SELECT logo_file FROM gdpr_company WHERE gdpr_company_id = ?", $page_form->id);
			$app->db->query("UPDATE gdpr_company SET logo = '', logo_file = '' WHERE gdpr_company_id = ?", $page_form->id);
			@unlink('lib/logo/'.$temp['logo_file']);
			@unlink('lib/logo/resized-'.$temp['logo_file']);
			unset($temp);
			unset($dataRecord['logo_img']);
			unset($dataRecord['logo_file']);
		}
		if($dataRecord['signature_img'] == '-1') {
			$temp=$app->db->queryOneRecord("SELECT signature_file FROM gdpr_company WHERE gdpr_company_id = ?", $page_form->id);
			$app->db->query("UPDATE gdpr_company SET signature = '', signature_file = '' WHERE gdpr_company_id = ?", $page_form->id);
			@unlink('lib/logo/'.$temp['signature_file']);
			@unlink('lib/logo/resized-'.$temp['signature_file']);
			unset($temp);
			unset($dataRecord['signature_img']);
			unset($dataRecord['signature_file']);
		} 
		$temp=array();
		$temp['logo'] = $dataRecord['logo_img'];
		$temp['logo_file'] = $dataRecord['logo_file'];
		$temp['signature'] = $dataRecord['signature_img'];
		$temp['signature_file'] = $dataRecord['signature_file'];
		$updates_rec=array();
		foreach($temp as $idx => $_temp) {
			if(trim($_temp) == '') {
				unset($temp[$idx]); 
			} else { 
				$updates_rec[] = $idx.' = ?'; 
				$update_params[] = $_temp; 
			}
		}
		if(!empty($updates_rec)) {
			$updates=implode(',', $updates_rec);
			$update_params[] = $page_form->id;
			$app->db->query('UPDATE gdpr_company SET ' . $updates . ' WHERE gdpr_company_id = ?', true, $update_params);
			if($dataRecord['logo_file'] != '') {
				rename('../temp/'.$dataRecord['logo_file'], 'lib/logo/'.$dataRecord['logo_file']);
				if($oldDataRecord['logo_file'] != '') {
					@unlink('lib/logo/'.$oldDataRecord['logo_file']);
					@unlink('lib/logo/resized-'.$oldDataRecord['logo_file']);
				}
			}
			if($dataRecord['signature_file'] != '') {
				rename('../temp/'.$dataRecord['signature_file'], 'lib/logo/'.$dataRecord['signature_file']);
				if($oldDataRecord['signature_file'] != '') {
					@unlink('lib/logo/'.$oldDataRecord['signature_file']);
					@unlink('lib/logo/resized-'.$oldDataRecord['signature_file']);
				}
			}
		}
	}

	function gdpr_update($event_name, $page_form) {
		global $app;

		$app->uses('gdpr_functions');
		
		include ISPC_WEB_PATH.'/gdpr/lib/lang/'.$_SESSION['s']['language'].'.lng';

		$dataRecord=$page_form->dataRecord;
		$oldDataRecord=$page_form->oldDataRecord;
		// admin
		if($app->auth->is_admin() && isset($dataRecord['client_group_id'])) {
			$date=date('Y-m-d');
			$client_group_id = $dataRecord['client_group_id'];
			$company_id = $dataRecord['company_id'];
			$updates = 'sys_groupid = ?, company_id = ?, signed = ?';
			$update_params = array($client_group_id, $company_id, $date);
			$update_params[] = $page_form->id;
			$app->db->query('UPDATE gdpr_contract SET ' . $updates . ', state = "pending" WHERE gdpr_contract_id = ?', true, $update_params);
		}
		// reseller
		elseif($app->auth->has_clients($_SESSION['s']['user']['userid']) && isset($dataRecord['client_group_id'])) {
			$client_group_id = $dataRecord['client_group_id'];
			$company_id = $dataRecord['company_id'];
			$updates = 'sys_groupid = ?, company_id = ?, sign_ip = ?, signed = ?';
			$update_params = array($client_group_id, $company_id, $_SERVER['REMOTE_ADDR'], $date);
			$update_params[] = $page_form->id;
			$app->db->query('UPDATE gdpr_contract SET ' . $updates . ' WHERE gdpr_contract_id = ?', true, $update_params);
		} 
		// client
		else {
			$company_id = $dataRecord['company_id'];
			$updates = 'company_id = ?, sign_ip = ?, state = "signed", signed = ?';
			$update_params = array($company_id, $_SERVER['REMOTE_ADDR'], $date);
			$update_params[] = $page_form->id;
			$app->db->query('UPDATE gdpr_contract SET ' . $updates . ' WHERE gdpr_contract_id = ?', true, $update_params);
		}
		// get the client id
		if(!$app->auth->is_admin() && !$app->auth->has_clients($_SESSION['s']['user']['userid'])) {
			$client_id = $_SESSION['s']['user']['client_id'];
		} else {
			$client = $app->db->queryOneRecord('SELECT client_id FROM sys_group WHERE groupid = ?', $client_group_id);
			$client_id = $client['client_id'];
			unset($client);
		}

		// Create the PDF
		$settings=$app->gdpr_functions->get_gdpr_settings();
		$date=date('Y-m-d H:i:s');
		$date=str_replace(' ','_',$date);
		$gdpr_filename=$app->gdpr_functions->create_filename($client_id, $date);
		$path_parts = pathinfo($gdpr_filename);
		$filename=$path_parts['basename'];
		$app->gdpr_functions->check_files($dataRecord['company_id']);
		$gdpr_pdf_template = $app->db->queryOneRecord("SELECT gdpr_pdf_template FROM gdpr_company WHERE gdpr_company_id = ?", $dataRecord['company_id'])['gdpr_pdf_template'];
		$pdf_gdpr_classname = 'PDFGDPR_'.$gdpr_pdf_template;
		if(!class_exists('PDFGDPR_'.$gdpr_pdf_template)) include 'templates/pdf_gdpr/'.$gdpr_pdf_template.'/make_pdf_gdpr.php';
		$pdf=new $pdf_gdpr_classname();
		$pdf->AliasNbPages();
		$pdf->createGDPRPDF($client_id, $dataRecord);
		$pdf_content = $pdf->Output('doc.pdf', 'S');
		if(is_file($gdpr_filename)) unlink($gdpr_filename);
		if(file_put_contents($gdpr_filename, $pdf_content)) {
			$updates = 'filename = ?, signed = ?';
			$update_params = array($filename, $date);
			$update_params[] = $page_form->id;
			$app->db->query('UPDATE gdpr_contract SET ' . $updates . ' WHERE gdpr_contract_id = ?', true, $update_params);
			if($settings['db_store'] == 'y') {
				$temp=base64_encode(file_get_contents($gdpr_filename));
				$app->db->query("UPDATE gdpr_contract SET contract = '$temp' WHERE gdpr_contract_id = ".intval($page_form->id));
			} else {
			}
		} else {
			$app->db->query('UPDATE gdpr_contract SET state = "pdf_failed" WHERE gdpr_contract_id = ?', $page_form->id);
		}
		unset($pdf);

		//* Send Notify Mail
		if(!$app->auth->is_admin() && (!$app->auth->has_clients($_SESSION['s']['user']['userid']) && isset($dataRecord['client_group_id']))) { //* Client		
			$company = $app->db->queryOneRecord("SELECT * FROM gdpr_company WHERE gdpr_company_id = ? AND active = 'y' AND enable_mail = 'y'", $dataRecord['company_id']);
			if(is_array($company) && !empty($company)) {
				$client=$app->db->queryOneRecord("SELECT * FROM client WHERE client_id = ?", $client_id);
				$message = str_replace('_CLIENT_', $client['contact_name'], $wb['notify_mail_text']);
				$app->gdpr_functions->mail($company['sender_email'], $wb['notify_mail_subject'], $message, $company['sender_email'], '', '', '', '', 'GDPR Plugin');
				unset($client);
			}
			unset($company);
		}
	}

	function gdpr_delete($event_name, $page_form) {
		global $app;

		$app->uses('gdpr_functions');
		
		include ISPC_WEB_PATH.'/gdpr/lib/lang/'.$_SESSION['s']['language'].'.lng';

		$dataRecord=$page_form->dataRecord;
		$client_id = $_SESSION['s']['user']['client_id'];
		//* Send Notify Mail
		if(!$app->auth->is_admin() && (!$app->auth->has_clients($_SESSION['s']['user']['userid']))) { 		
			$company = $app->db->queryOneRecord("SELECT * FROM gdpr_company WHERE gdpr_company_id = ? AND active = 'y' AND enable_mail = 'y'", $dataRecord['company_id']);
			if(is_array($company) && !empty($company)) {
				$client=$app->db->queryOneRecord("SELECT * FROM client WHERE client_id = ?", $client_id);
				$message = str_replace('_CLIENT_', $client['contact_name'], $wb['notify_mail_cancel_text']);
				$app->gdpr_functions->mail($company['sender_email'], $wb['notify_mail_cancel_subject'], $message, $company['sender_email'], '', '', '', '', 'GDPR Plugin');
				unset($client);
			}
			unset($company);
		}
	}
	
	function gdpr_data($event_name, $page_form) {
		global $app;

		$app->uses('gdpr_functions');

		$dataRecord=$page_form->dataRecord;
		$client_group_id = $page_form->dataRecord['client_group_id'];
		if($app->auth->is_admin()) {
			$temp = $app->db->queryOneRecord('SELECT client_id, userid FROM sys_user WHERE default_group = ?', $client_group_id);
			$client_id = $temp['client_id'];
			$sys_userid = $temp['userid'];
			unset($temp);
		} else {
			$client_id = $page_form->dataRecord['client_id'];
		}

		$date=date('Y-m-d H:i:s');

		$outformat=$dataRecord['type_select'];
		$settings=$app->gdpr_functions->get_gdpr_settings();
		$gdpr_filename=$app->gdpr_functions->create_filename($client_id, $date, 'export.'.$outformat);
		$path_parts = pathinfo($gdpr_filename);
		$filename=$path_parts['basename'];
		$temp = $app->db->queryOneRecord("SELECT gdpr_company_id, gdpr_stored_pdf_template, gdpr_stored_xml_template FROM gdpr_company WHERE gdpr_company_id = (SELECT gdpr_company_id FROM gdpr_client_settings WHERE client_id = ?)", $client_id);
		$app->gdpr_functions->check_files($temp['gdpr_company_id']);
		$gdpr_pdf_template = $temp['gdpr_stored_pdf_template'];
		$gdpr_xml_template = $temp['gdpr_stored_xml_template'];
		unset($temp);

		if($outformat == 'pdf') {		
			// Create the PDF
			$pdf_gdpr_classname = 'PDFSTOREDDATA_'.$gdpr_pdf_template;
			if(!class_exists('PDFSTOREDDATA_'.$gdpr_pdf_template)) include 'templates/pdf_stored_data/'.$gdpr_pdf_template.'/make_pdf_stored_data.php';
			$pdf=new $pdf_gdpr_classname();
			$pdf->AliasNbPages();
			$pdf->createSTOREDDATAPDF($client_id, $dataRecord);
			$content = $pdf->Output('doc.pdf', 'S');
			if($app->auth->is_admin()) {
				$updates = 'sys_userid = ?, sys_groupid = ?, client_id = ?, filename = ?, sys_perm_user = "", sys_perm_group = ""';
				$update_params = array($sys_userid, $client_group_id, $client_id, $filename);
			} else {
				$updates = 'filename = ?';
				$update_params = array($filename);
			}
		} elseif($outformat == 'xml') {	
			$xml_gdpr_classname = 'XMLSTOREDDATA_'.$gdpr_xml_template;
			if(!class_exists('XMLSTOREDDATA_'.$gdpr_xml_template)) include 'templates/xml_stored_data/'.$gdpr_xml_template.'/make_xml_stored_data.php';
			$xml=new $xml_gdpr_classname();
			$content = $xml->createSTOREDDATAXML($client_id);
			if($app->auth->is_admin()) {
				$updates = 'sys_userid = ?, sys_groupid = ?, client_id = ?, filename = ?, filetype = ?, sys_perm_user = "", sys_perm_group = ""';
				$update_params = array($sys_userid, $client_group_id, $client_id, $filename, 'text/xml');
			} else {
				$updates = 'filename = ?, filetype = ?';
				$update_params = array($filename, 'text/xml');
			}
		} elseif($outformat == 'xml-full') {	
			$xml_gdpr_classname = 'XMLSTOREDDATA_'.$gdpr_xml_template;
			if(!class_exists('XMLSTOREDDATA_'.$gdpr_xml_template)) include 'templates/xml_stored_data/'.$gdpr_xml_template.'/make_xml_stored_data.php';
			$xml=new $xml_gdpr_classname();
			$content = $xml->createSTOREDDATAXML($client_id, $client_group_id);
			if($app->auth->is_admin()) {
				$updates = 'sys_userid = ?, sys_groupid = ?, client_id = ?, filename = ?, filetype = ?, sys_perm_user = "", sys_perm_group = ""';
				$update_params = array($sys_userid, $client_group_id, $client_id, $filename, 'text/xml');
			} else {
				$updates = 'filename = ?, filetype = ?';
				$update_params = array($filename, 'text/xml');
			}
		} else {
			die("format error");
		}

		if(is_file($gdpr_filename)) unlink($gdpr_filename);
		if(file_put_contents($gdpr_filename, $content)) {
			$update_params[] = $page_form->id;
			$app->db->query('UPDATE gdpr_client_data SET ' . $updates . ' WHERE gdpr_client_data_id = ?', true, $update_params);
		} else {
			die("error");
		}
	}

	function gdpr_admin_update($event_name, $page_form) {
		global $app;

		$app->uses('gdpr_functions');
		$date=date('Y-m-d');
		$dataRecord=$page_form->dataRecord;
		$oldDataRecord=$page_form->oldDataRecord;
		if($dataRecord['state'] != $oldDataRecord['state']) {
			if ($dataRecord['state'] == 'signed') {
				$updates = 'signed = ?';
				$update_params = array($date);
			}
			if ($dataRecord['state'] == 'pending') {
				$updates = 'signed = ?, sign_ip = ?, deleted = ?, del_ip = ?, deleted = ?';
				$update_params = array('0000-00-00 00:00:00', '', '0000-00-00 00:00:00', '');
			}
			if ($dataRecord['state'] == 'deleted') {
				$updates = 'deleted = ?';
				$update_params = array($date);
			}
			$update_params[] = $page_form->id;
			$app->db->query('UPDATE gdpr_contract SET ' . $updates . ' WHERE gdpr_contract_id = ?', true, $update_params);
		}
	}

	function client_before_delete($event_name, $page_form) {
		global $app;

		include ISPC_WEB_PATH.'/gdpr/lib/lang/'.$_SESSION['s']['language'].'.lng';

		$dataRecord=$page_form->dataRecord;

		$in_use = false;

		$temp=$app->db->queryAllRecords('SELECT client_id FROM gdpr_client_data WHERE client_id = ?', $dataRecord['client_id']);
        if(is_array($temp) && !empty($temp)) $in_use = true;

        if(!$in_use){
			$temp=$app->db->queryAllRecords('SELECT sys_groupid FROM gdpr_contract WHERE sys_groupid = ?', $dataRecord['sys_groupid']);
        	if(is_array($temp) && !empty($temp)) $in_use = true;
        }

		if($in_use) $app->error($wb['gdpr_client_has_data_txt']);

	}

	function client_update($event_name, $page_form) {
		global $app;

		$rec = $app->db->queryOneRecord('SELECT client_id FROM gdpr_client_settings WHERE client_id = ?', $page_form->id);
		$client = $app->db->queryOneRecord('SELECT * FROM client where client_id = ?', $page_form->id);
		$company=1;
		if($client['sys_groupid'] != 1) {
			$_company=$app->db->queryOneRecord('SELECT gdpr_company_id FROM gdpr_company WHERE sys_groupid = ?', $client['sys_groupid'])['gdpr_company_id'];
			if(isset($_company) && !empty($_company)) $company=$_company;
		}
		$app->db->query('UPDATE gdpr_client_settings SET sys_userid = ?, sys_groupid = ?, sys_perm_user = ?, sys_perm_group = ?, sys_perm_other = ?, gdpr_company_id = ? WHERE client_id = ?', $client['sys_userid'], $client['sys_groupid'], $client['sys_perm_user'], $client['sys_perm_group'],  $client['sys_perm_other'], $company, $page_form->id);
	}

	function client_insert($event_name, $page_form) {
		global $app, $conf;

		$app->uses('gdpr_functions');

		$rec = $app->db->queryOneRecord('SELECT client_id FROM gdpr_client_settings WHERE client_id = ?', $page_form->id);
		if(!isset($rec['client_id'])) {
			$client = $app->db->queryOneRecord('SELECT * FROM client where client_id = ?', $page_form->id);
			$company = $app->db->queryOneRecord("SELECT gdpr_company_id FROM gdpr_company WHERE `primary` = 'y'")['gdpr_company_id'];
			if(empty($company)) $company = $app->db->queryOneRecord("SELECT gdpr_company_id FROM gdpr_company ORDER BY gdpr_company_id")['gdpr_company_id'];
			if($client['sys_groupid'] != 1) {
				$_company = $app->db->queryOneRecord("SELECT gdpr_company_id FROM gdpr_company WHERE `primary` = 'y' AND sys_groupid = ?", $client['sys_groupid'])['gdpr_company_id'];
				if(isset($_company) && !empty($_company)) $company=$_company;
			}
			$sql = "INSERT INTO gdpr_client_settings (client_id, sys_userid, sys_groupid, sys_perm_user, sys_perm_group, sys_perm_other, gdpr_company_id, gdpr_email)
			VALUES (?, ?, ?, ?, ?, ?, ?, '')";
			$app->db->query($sql, $page_form->id, $client['sys_userid'], $client['sys_groupid'], $client['sys_perm_user'], $client['sys_perm_group'], $client['sys_perm_other'], $company);

        	$modules = $conf['interface_modules_enabled'];
			$rec=$app->gdpr_functions->get_gdpr_settings();
			if($rec['add_client'] == 'y') $app->db->query("UPDATE sys_user SET modules = ? WHERE client_id = ?", $modules.',gdpr', $page_form->id);
		}
	}

	function client_delete($event_name, $page_form) {
		global $app;

		$app->db->query("DELETE FROM gdpr_client_settings WHERE client_id = ?", $page_form->id);

	}
		
}
?>
