<?php

require('lib/tfpdf/tfpdf.php');

class FPDF_Extensions extends tFPDF {
	var $angle=0;

	function Header() {
		if(!$this->no_header) {
			global $app;
			$app->uses('gdpr_functions');

        	//Put the watermark
	        $this->SetFont($this->font_name, 'B', 50);
    	    $this->SetTextColor($this->watermark_colour);

			$max_watermark = 34;
			$watermark = $this->lng($this->watermark);

			if($this->repeat_watermark) {
				$length = strlen($watermark);
				$i = $max_watermark-$length;
				$i = $i / $length;
				$temp='';
				for ($k = 0; $k <= $i; $k++) $temp = $temp.$watermark.' ';
				$length=$max_watermark - strlen($temp);
				if ($length > 0) $t=substr($watermark, 0, $length-2); else $t='';
			    $temp=$temp." ".$t;
				$watermark=substr($temp, 0, $max_watermark-1);
			}
			if($this->center_watermark) $watermark = str_pad($watermark, $max_watermark, " ", STR_PAD_LEFT);

			$this->RotatedText(25,300,$watermark,55);

	        $this->SetMargins(15, 20, 15);
			$heigth=10;
        	if($this->logo_file != '') {
				$temp=$app->gdpr_functions->resize_image($this->logo_file, $this->img_path, $this->logo_width, $this->logo_height);
				if($temp !== false) {
					$this->logo_file = $temp;
					$dimensions=$app->gdpr_functions->image_dimensions($this->logo_file, $this->img_path);
					$width=$dimensions['width'];
					$heigth=$dimensions['heigth'];
					$this->Image($this->img_path.$this->logo_file, 167 - $width, 10);
				}
			}
			$this->setXY(15, round($heigth));
		}
	}

    function Footer() {
    	$company = $this->company;
        $this->Line(15, 266, 200, 266);
        $this->SetFont($this->font_name, '', 7);
        $cell_heights = array();
        $column1_text = '';
        $column1_text .= $company['company_name']."\n";
        if($company['ceo_name'] != '') $column1_text .= $this->lng('ceo_txt').': '.$company['ceo_name']."\n";
		if($company['company_register'] != '') $column1_text .= $this->lng('company_register_txt').': '.$company['company_register']."\n";
		if($company['vat_id'] == '') {
	    	$column1_text .= $this->lng('tax_id_txt').': '.$company['tax_id']."\n";
		} else {
	        $column1_text .= $this->lng('vat_id_txt').': '.$company['vat_id']."\n";
		}
        $cell_heights[] = $this->NbLines(50, $column1_text);
        $column2_text = '';
        $column2_text .= $company['street']." | ".$company['zip']." ".$company['city']."\n";
        $column2_text .= $this->lng('telephone_txt').': '.$company['telephone']."\n";
        if($company['fax'] != '') $column2_text .= $this->lng('fax_txt').': '.$company['fax']."\n";
        if($company['email'] != '') $column2_text .= $company['email'];
        if($company['internet'] != '') $column2_text .= " | ".$company['internet']."\n"; else $column2_text .= "\n";
        $cell_heights[] = $this->NbLines(50, $column2_text);
        $column3_text = '';
        $column3_text .= $this->lng('bank_account_owner_txt').': '.$company['bank_account_owner']."\n";
        $column3_text .= $this->lng('bank_name_txt').': '.$company['bank_name']."\n";
        $column3_text .= $this->lng('bank_account_iban_txt').': '.$company['bank_account_iban']."\n";
        if($company['bank_account_swift'] != '') $column3_text .= $this->lng('bank_account_swift_txt').': '.$company['bank_account_swift']."\n";
        $cell_heights[] = $this->NbLines(50, $column3_text);
        $cell_height = max($cell_heights) - 1;
        $this->SetXY(15, -30);
        $this->MultiCell(60, $cell_height, $column1_text, 0, 'L');
        $this->SetXY(78, -30);
        $this->MultiCell(160, $cell_height, $column2_text, 0, 'L');
        $this->SetXY(155, -30);
        $this->MultiCell(50, $cell_height, $column3_text, 0, 'L');
        $this->SetY(-5);
        $this->Cell(0, -2, $this->lng('page_txt').' '.$this->PageNo().' / {nb}', 0, 0, 'C');
    }

	function write_logo($heigth = 10) {
		global $app;
		if($this->no_header) {
			if($this->logo_file != '') {
				$temp=$app->gdpr_functions->resize_image($this->logo_file, $this->img_path, $this->logo_width, $this->logo_height);
				if($temp !== false) {
					$this->logo_file = $temp;
					$dimensions=$app->gdpr_functions->image_dimensions($this->logo_file, $this->img_path);
					$width=$dimensions['width'];
					$heigth=$dimensions['heigth'];
					$this->Image($this->img_path.$this->logo_file, 167 - $width, 10);
				}
			}
			$this->setXY(15, round($heigth));
		}
	}

    //* Internal helper function: set array with column widths
    function SetWidths($w) {
        $this->widths=$w;
    }

    //* Internal helper function: set array with column aligns
    function SetAligns($a) {
        $this->aligns=$a;
    }


    //* Internal helper function: write details row
    function details_row($fields, $value, $title='', $unset=false) {
        //* Title
        if ($title != '') {
            $this->SetFont($this->font_name, 'B', 10);
            $this->Cell(0,5, $this->lng($title),0,0,'C');
            $this->Ln();
            $this->SetFont($this->font_name, '', 10);
        }
        $data=array();
        foreach ($fields as $temp) {
            $data[$temp]=$value[$temp];
            if($unset) {
                unset($value[$temp]);
            }
        }
        if (is_array($data)) {
            foreach ($data as $key_name => $key_value) {
                $tmp=array();
                $tmp[0]=$this->lng($key_name);
                $tmp[1]=trim($key_value);
                $this->Row($tmp);
            }
        }
        //* Add newline
        $this->Ln();

        return $value;
    }

    //* Internal helper function: write table row
    function Row($data) {
        //Calculate the height of the row
        $nb=0;
        for($i=0;$i<count($data);$i++) {
            $nb=max($nb, $this->NbLines($this->widths[$i], $data[$i]));
        }
        $h=5*$nb;
        //Issue a page break first if needed
        $this->CheckPageBreak($h);
        //Draw the cells of the row
        for($i=0;$i<count($data);$i++) {
            $w=$this->widths[$i];
            $a=isset($this->aligns[$i]) ? $this->aligns[$i] : 'L';
            //Save the current position
            $x=$this->GetX();
            $y=$this->GetY();
            //Draw the border
            $this->Rect($x, $y, $w, $h);
            //Print the text
            $this->MultiCell($w, 5, $data[$i], 0, $a);
            //Put the position to the right of the cell
            $this->SetXY($x+$w, $y);
        }
        //Go to the next line
        $this->Ln($h);
    }

    //* Internal helper function: check for page breaks
    function CheckPageBreak($h) {
        if($this->GetY()+$h>$this->PageBreakTrigger)
            $this->AddPage($this->CurOrientation);
    }

    //* Internal helper function: Computes the number of lines a MultiCell of width w will take
    function NbLines($w, $txt) {
        $cw=&$this->CurrentFont['cw'];
        if($w==0)
            $w=$this->w-$this->rMargin-$this->x;
        $wmax=($w-2*$this->cMargin)*1000/$this->FontSize;
        $s=str_replace("\r", '', $txt);
        $nb=strlen($s);
        if($nb>0 and $s[$nb-1]=="\n")
            $nb--;
        $sep=-1;
        $i=0;
        $j=0;
        $l=0;
        $nl=1;
        $unicode = $this->UTF8StringToArray($txt);
        foreach($unicode as $c) {
            if($s[$i] =="\n") {
                $i++;
                $sep=-1;
                $j=$i;
                $l=0;
                $nl++;
                continue;
            }
            if($s[$i] ==' ')
                $sep=$i;
            if (isset($cw[$c])) { $l += (ord($cw[2*$c])<<8) + ord($cw[2*$c+1]); }
            else if($c>0 && $c<128 && isset($cw[chr($c)])) { $l += $cw[chr($c)]; }
            else if(isset($this->CurrentFont['desc']['MissingWidth'])) { $l += $this->CurrentFont['desc']['MissingWidth']; }
            else if(isset($this->CurrentFont['MissingWidth'])) { $l += $this->CurrentFont['MissingWidth']; }
            else { $l += 500; }

            if($l>$wmax) {
                if($sep==-1) {
                    if($i==$j)
                        $i++;
                } else
                    $i=$sep+1;
                $sep=-1;
                $j=$i;
                $l=0;
                $nl++;
            } else
                $i++;
        }

        return $nl;
    }

    //* Internal helper function: to decode text, deprecated
    function textdecode($records) {
        return $records;
    }

	function Rotate($angle,$x=-1,$y=-1) {
		if($x==-1)
			$x=$this->x;
		if($y==-1)
			$y=$this->y;
		if($this->angle!=0)
			$this->_out('Q');
		$this->angle=$angle;
		if($angle!=0) {
			$angle*=M_PI/180;
			$c=cos($angle);
			$s=sin($angle);
			$cx=$x*$this->k;
			$cy=($this->h-$y)*$this->k;
			$this->_out(sprintf('q %.5F %.5F %.5F %.5F %.2F %.2F cm 1 0 0 1 %.2F %.2F cm',$c,$s,-$s,$c,$cx,$cy,-$cx,-$cy));
		}
	}

	function _endpage() {
		if($this->angle!=0) {
			$this->angle=0;
			$this->_out('Q');
		}
		parent::_endpage();
	}

	function RotatedText($x, $y, $txt, $angle) {
		//Text rotated around its origin
		$this->Rotate($angle,$x,$y);
		$this->Text($x,$y,$txt);
		$this->Rotate(0);
	}

	function lng($text) {
        global $app, $conf;
        if(!is_array($this->wb)) {
			if(file_exists(ISPC_WEB_PATH.'/gdpr/lib/lang/'.$this->client['language'].'_gdpr_pdf.lng')) 
	           include ISPC_WEB_PATH.'/gdpr/lib/lang/'.$this->client['language'].'_gdpr_pdf.lng';
			else
				include ISPC_WEB_PATH.'/gdpr/lib/lang/'.$this->company['default_lng'].'_gdpr_pdf.lng';
            $this->wb = $wb;
        }

        if(isset($this->wb[$text])) {
            return $this->textdecode($this->wb[$text]);
        } else {
            return $this->textdecode($text);
        }
    }

	function set_style($new, $add) {
		if ($add === true) {
			if (strpos($this->font_style, $new) !== false) {
				
			} else {
				$this->font_style=$this->font_style.''.$new;
			}
		} else {
			$this->font_style=str_replace($new,'',$this->font_style);
		}
	}

	function set_font_size($new, $add) {
		if ($add === true) {
			if($this->font_size < $new) {
				$this->font_size = $new;
			} else {
			}
		} else {
			$this->font_size = $this->default_font_size;
		}
	}

	function write_line($line) {	
		global $app;
		if (strpos($line, '{PAGEBREAK}') !== false) {
			$line=str_replace('{PAGEBREAK}','',$line);
			$this->AddPage();
		}
		if (strpos($line, '{CUSTOMER}') !== false) {
			$line=str_replace('{CUSTOMER}',$this->customer,$line);
		}
		if (strpos($line, '{SIGNATURE}') !== false) {
			$line=str_replace('{SIGNATURE}','',$line);
			if(!$this->preview_client) {
				//* Add the signature image
        		if($this->signature_file != '') {
					$temp=$app->gdpr_functions->resize_image($this->signature_file, $this->img_path, $this->signature_width, $this->signature_height);
					if($temp !== false) {
						$this->signature_file = $temp;
						$dimensions=$app->gdpr_functions->image_dimensions($this->signature_file, $this->img_path);
						$width=$dimensions['width'];
						$heigth=$dimensions['heigth'];
						$this->Image($this->img_path.$this->signature_file, null, null, 120);
					}
				}
			}
		}

		// Fonts
		if (strpos($line, '{BOLD}') !== false || strpos($line, '{UN_BOLD}') !== false) {
			if (strpos($line, '{UN_BOLD}') !== false) {
				$this->set_style('B', true);
				$line=str_replace(array('{BOLD}','{UN_BOLD}'),'',$line);
				$this->SetFont($this->font_name, $this->font_style, $this->font_size);
				$this->Write(5, $line, '', 0, '', false, 0, false, false, 0);$this->ln();
				$this->set_style('B', false);
				$this->SetFont($this->font_name, $this->font_style, $this->font_size);
			} else {
				$line=str_replace(array('{BOLD}'),'',$line);
				$this->set_style('B', true);
				$this->SetFont($this->font_name, $this->font_style, $this->font_size);
				$this->Write(5, $line, '', 0, '', false, 0, false, false, 0);$this->ln();
			}
		} elseif (strpos($line, '{HEADING1}') !== false || strpos($line, '{UN_HEADING1}') !== false) {
			if (strpos($line, '{UN_HEADING1}') !== false) {
				$this->set_style('B', true);
				$this->set_font_size($this->font_size_h1, true);
				$line=str_replace(array('{HEADING1}','{UN_HEADING1}'),'',$line);
				$this->SetFont($this->font_name, $this->font_style, $this->font_size);
				$this->Write(5, $line, '', 0, '', false, 0, false, false, 0);$this->ln($this->font_size);
				$this->set_style('B', false);
				$this->set_font_size($this->font_size_h1, false);
				$this->SetFont($this->font_name, $this->font_style, $this->font_size);
			} else {
				$this->set_style('B', true);
				$this->set_font_size($this->font_size_h1, true);
				$line=str_replace(array('{HEADING1}'),'',$line);
				$this->SetFont($this->font_name, $this->font_style, $this->font_size);
				$this->Write(5, $line, '', 0, '', false, 0, false, false, 0);$this->ln($this->font_size);
			}
		} elseif (strpos($line, '{HEADING2}') !== false || strpos($line, '{UN_HEADING2}') !== false) {
			if (strpos($line, '{UN_HEADING2}') !== false) {
				$this->set_style('B', true);
				$this->set_font_size($this->font_size_h2, true);
				$line=str_replace(array('{HEADING2}','{UN_HEADING2}'),'',$line);
				$this->SetFont($this->font_name, $this->font_style, $this->font_size);
				$this->Write(5, $line, '', 0, '', false, 0, false, false, 0);$this->ln($this->font_size);
				$this->set_style('B', false);
				$this->set_font_size($this->font_size_h2, false);
				$this->SetFont($this->font_name, $this->font_style, $this->font_size);
			} else {
				$this->set_style('B', true);
				$this->set_font_size($this->font_size_h2, true);
				$line=str_replace(array('{HEADING2}'),'',$line);
				$this->SetFont($this->font_name, $this->font_style, $this->font_size);
				$this->Write(5, $line, '', 0, '', false, 0, false, false, 0);$this->ln($this->font_size);
			}
		} elseif (strpos($line, '{UNDERLINE}') !== false || strpos($line, '{UN_UNDERLINE}') !== false) {
			if (strpos($line, '{UN_UNDERLINE}') !== false) {
				$this->set_style('U', true);
				$line=str_replace(array('{UNDERLINE}','{UN_UNDERLINE}'),'',$line);
				$this->ln(5);
				$this->SetFont($this->font_name, $this->font_style, $this->font_size);
				$this->Write(5, $line, '', 0, '', false, 0, false, false, 0);$this->ln();
				$this->set_style('U', false);
				$this->SetFont($this->font_name, $this->font_style, $this->font_size);
			} else {
				$line=str_replace(array('{UNDERLINE}'),'',$line);
				$this->set_style('U', true);
				$this->ln(5);
				$this->SetFont($this->font_name, $this->font_style, $this->font_size);
				$this->Write(5, $line, '', 0, '', false, 0, false, false, 0);$this->ln();
			}				
		} else {
			$this->Write(5, $line, '', 0, '', false, 0, false, false, 0);
			if($this->font_size != $this->default_font_size) $this->ln($this->font_size); else $this->ln();
		}
	}

	function build_placeholder() {
		global $app;

		$dataRecord = $this->dataRecord;
		// Build placeholder {CUSTOMER}
		$temp=array();
		$client_name=@($this->client['contact_firstname'] != '')?$this->client['contact_firstname'].' '.$this->client['contact_name']:$this->client['contact_name'];
		if($this->client['company_name'] != '') {
			$temp[] = $this->client['company_name'];
			if($this->client_settings['gdpr_show_contact_name'] == 'y') $temp[] = $client_name; else {}
		} else {
			$temp[] = $client_name;
		}
		if(trim($app->gdpr_functions->get_address($this->client)) != '') $temp[] = $app->gdpr_functions->get_address($this->client, $this->foreign_language);
		$this->customer=implode(', ', $temp);
		unset($temp);
		unset($client_name);

		// Build placeholder {COMPANY}
		$temp=array();
		$temp[]=$this->company['company_name'];
		if(trim($app->gdpr_functions->get_address($this->company)) != '') $temp[] = $app->gdpr_functions->get_address($this->company, $this->foreign_language);
		$this->_company=implode(', ', $temp);
		unset($temp);

		// Build placeholder {CONCERNED_DATA}
		$this->translate_personal='';
		$temp=array(
			'gdpr_personal_data',
			'gdpr_communictaion_data',
			'gdpr_contract_data',
			'gdpr_log_data'
		);
		foreach($temp as $data) {
			if($this->preview_client) {
				$this->translate_personal.='- '.$this->lng($data)."\n";
			} else {
				if(isset($dataRecord[$data]) && $dataRecord[$data]==1) $this->translate_personal.='- '.$this->lng($data)."\n";
			}
		}
		unset($temp);
		if(is_array($dataRecord['gdpr_additional']) && !empty($dataRecord['gdpr_additional'])) 
			foreach($dataRecord['gdpr_additional'] as $data) if(trim($data) != '') $this->translate_personal.='- '.$data."\n";
		if($this->translate_personal!='') 
			$this->translate_personal=$this->lng('gdpr_personal_data_txt')."\n".$this->lng('gdpr_personal_desc_txt')."\n".$this->translate_personal;
		else 
			$this->translate_personal=$this->lng('gdpr_personal_data_txt')."\n".$this->lng('none_txt')."\n";

		// Build placeholder {CONTRACT_DATA}
		$this->translate_concerned='';
		$temp=array(
			'concerned_1',
			'concerned_2'
		);
		foreach($temp as $data) {
			if($this->preview_client) {
				$this->translate_concerned.='- '.$this->lng('gdpr_'.$data)."\n";
			} else {
				if(isset($dataRecord[$data]) && $dataRecord[$data]==1) $this->translate_concerned.='- '.$this->lng('gdpr_'.$data)."\n";
			}
		}
		unset($temp);
		if(is_array($dataRecord['concerned_additional']) && !empty($dataRecord['concerned_additional'])) 
			foreach($dataRecord['concerned_additional'] as $data) if(trim($data) != '') $this->translate_concerned.='- '.$data."\n";
		if($this->translate_concerned!='') 
			$this->translate_concerned=$this->lng('gdpr_concerned_data_txt')."\n".$this->lng('gdpr_concerned_desc_txt')."\n".$this->translate_concerned;
		else
			$this->translate_concerned=$this->lng('gdpr_concerned_data_txt')."\n".$this->lng('none_txt')."\n";
	}

	function translate($template, $values, $trans) {
		global $app;
		$template=$app->gdpr_functions->translate_text($trans, $values, $template);
		return(explode(PHP_EOL, $template));
	}

	function set_pdf() {
		global $app;

		if($this->preview && $this->client_id == 1) {
			$this->foreign_language=false;
	    	$this->client['city'] = $this->company['city'];
		} else {
			$this->client=$app->db->queryOneRecord("SELECT * FROM client WHERE client_id = ?", $this->client_id);
			$this->client_company = $client['company_name'];
		}
		$this->client_settings=$app->db->queryOneRecord("SELECT gdpr_show_contact_name FROM gdpr_client_settings WHERE client_id = ?", $this->client_id);

		if($this->preview_client) {
			//* Add watermark
			$this->center_watermark = $this->config['center_watermark'];
			$this->repeat_watermark = $this->config['repeat_watermark'];
			$this->watermark_colour = $this->config['watermark_colour'];
			$this->watermark = 'watermark_txt';
		}

		$this->foreign_language=@($this->client['country'] == $this->company['country'])?false:true;

		$this->preview_client = @($preview && $this->client_id != 1)?true:false;

		// Logo
		$this->logo_width = $this->config['max_logo_width'];
		$this->logo_height = $this->config['max_logo_height'];
        if(is_file($this->img_path.$this->company['logo_file'])) $this->logo_file = $this->company['logo_file'];
		// Signature
		$this->signature_width = $this->config['max_signature_width'];
		$this->signature_height = $this->config['max_signature_height'];
        if(is_file($this->img_path.$this->company['signature_file'])) $this->signature_file = $this->company['signature_file'];

		//* Set fonts
		$this->font_name = $this->config['font_name'];
		$this->font_normal_file = $this->config['font_normal_file'];
		$this->font_bold_file = $this->config['font_bold_file'];
		//* Set font size
		$this->font_size = $this->config['font_size'];
		$this->font_size_h1 = $this->config['font_size_h1'];
		$this->font_size_h2 = $this->config['font_size_h2'];
		$this->default_font_size = $this->font_size;
		//* Add the font files
		$this->AddFont($this->font_name, '', $this->font_normal_file, true);
		$this->AddFont($this->font_name, 'B', $this->font_bold_file, true);

		//* Data for the PDF
		$this->SetTitle($this->lng('pdf_title_txt').' '.$this->company['company_name']);
		$this->SetAuthor($this->company['company_name']);
		$this->SetCreator("ISPConfig GDPR Plugin by schaal @it UG");
		$this->SetFillColor($this->config['gdpr_details_fill_color']);
		$this->SetAutoPageBreak(true,35);
		$this->AddPage();
		$this->SetMargins(15, 20, 15);

		$this->write_logo();

        $this->SetWidths(explode(',', $this->config['gdpr_column_widths']));
		$this->SetAligns(explode(',', $this->config['gdpr_column_aligns']));
		$this->SetFont($this->font_name, '', $this->font_size);
		$font_size = $this->font_size;
		$this->font_style = '';
	}
	
	function get_template() {
		global $app;

		$dataRecord = $this->dataRecord;

		$this->template_lng=$app->db->queryOneRecord("SELECT lng FROM gdpr_templates WHERE master = 'y' AND company_id = ? AND gdpr_template_id = ?", $dataRecord['company_id'], $dataRecord['template_id'])['lng'];
		if($this->preview && $this->client_id == 1) $this->client['language'] = $this->template_lng;
		$temp=$app->db->queryOneRecord("SELECT text, company_id FROM gdpr_templates WHERE gdpr_template_id = ?", $dataRecord['template_id']);
		$template=$temp['text'];
		$template=$app->gdpr_functions->merge_templates($template);
		if($this->preview && $this->client_id == 1) {
	    	$this->client['language'] = $this->template_lng;
		}
		return $template;
	}

	function prepare_pdf() {
		$this->set_pdf();
		$this->build_placeholder();
	}

	function data2pdf($export, $type, $simple=false) {
		$details_array=array();
		$data=array();
		foreach($export as $idx=>$val) $details_array[] = $idx;
		if(!$simple) {
			$this->set_style('B', true);
			$this->set_font_size($this->font_size_h1, true);
			$this->SetFont($this->font_name, $this->font_style, $this->font_size);
			$this->Write(5, $this->lng($type), '', 0, '', false, 0, false, false, 0);
			$this->ln($this->font_size);
			$this->set_style('B', false);
			$this->set_font_size($this->font_size_h1, false);
			$this->SetFont($this->font_name, $this->font_style, $this->font_size);
			$this->details_row($details_array, $export);
			$this->ln($this->font_size);
		} else {
			$this->details_row($details_array, $export);
			$this->ln($this->font_size);
		}
    }

	function write_stored_domains($domains) {
		if(is_array($domains) &&!empty($domains)) {
			$this->SetWidths(explode(',', $this->config['stored_data_column_widths']));
			$this->SetAligns(explode(',', $this->config['stored_data_aligns']));
			$this->set_style('B', true);
			$this->set_font_size($this->font_size_h1, true);
			$this->SetFont($this->font_name, $this->font_style, $this->font_size);
			$this->Write(5, $this->lng('domains'), '', 0, '', false, 0, false, false, 0);
			$this->ln($this->font_size);
			$this->set_style('B', false);
			$this->set_font_size($this->font_size_h1, false);
			$this->SetFont($this->font_name, $this->font_style, $this->font_size);
			$this->SetFont($this->font_name, '', 8);
			foreach($domains as $domain) {
				$tmp = array();
				$tmp[0] = $domain['domain'];
				$this->Row($tmp);
			}
			$this->ln($this->font_size);
		}
	}

	function write_billing($invoices) {
		if(is_array($invoices) && !empty($invoices)) {
			$this->set_style('B', true);
			$this->set_font_size($this->font_size_h1, true);
			$this->SetFont($this->font_name, $this->font_style, $this->font_size);
			$this->Write(5, $this->lng('invoices_txt'), '', 0, '', false, 0, false, false, 0);
			$this->ln($this->font_size);
			$this->set_style('B', false);
			$this->set_font_size($this->font_size_h1, false);
			$this->SetFont($this->font_name, $this->font_style, $this->font_size);
			foreach($invoices as $idx=>$val) {
				$out=array();
				foreach($val as $i=>$v) $out[$i]=$v;
				$this->data2pdf($out, '', true);
				unset($out);
			}
		}
	}

}
?>
