<?php

/*
   Here we extend tfpdf and create a new class. the name of the Class has to be
   XMLSTOREDDATA_ plus the name of the template, in this case "default" so the full
   name of the class is "XMLSTOREDDATA_default".
*/

class XMLSTOREDDATA_default {

	function generate_xml_element($dom, $data) {

		if (empty( $data['name'])) return false;
 
		$element_value = (!empty($data['value']))?$data['value']:null;
		$element = $dom->createElement($data['name'], $element_value);
 
		if (!empty($data['attributes']) && is_array($data['attributes'])) 
			foreach ($data['attributes'] as $attribute_key => $attribute_value) $element->setAttribute($attribute_key, $attribute_value );
 
		foreach ($data as $data_key => $child_data) {
			if (!is_numeric($data_key)) continue;
			$child = $this->generate_xml_element($dom, $child_data);
			if ($child) $element->appendChild( $child );
		}
		return $element;
	}

	function add_xml($xml_data, $rec, $type) {

		if(is_array($rec) && !empty($rec)) {
			$out=array('name' => $type);
			foreach($rec as $idx=>$val) array_push($out, array('name'=>$idx, 'value'=>$val));
			array_push($xml_data, $out);
		}
		return($xml_data);
	}

	function createSTOREDDATAXML($client_id, $client_group_id = 0) {
		global $app;

		$app->uses('gdpr_functions_data');
		$company=$app->db->queryOneRecord("SELECT * FROM gdpr_company WHERE gdpr_company_id = (SELECT gdpr_company_id FROM gdpr_client_settings WHERE client_id = ?)", $client_id);
		$xml_data = array('name' => 'data-export','attributes' => array('company' => 'schaal @it UG','date' => time()));

		if($client_group_id > 1 && $client_id > 0) { //* Full XML
			//* ISPConfig
			$tables = 'cron,client,dns_rr,dns_soa,dns_slave,ftp_user,mail_access,mail_content_filter,mail_domain,mail_forwarding,mail_get,mail_user,mail_user_filter,shell_user,spamfilter_users,support_message,web_database,web_database_user,web_domain,web_traffic,domain';
			$tables_array = explode(',', $tables);
			$table_list = array();
			foreach($tables_array as $table) {
				if($table != '') {
					$records = $app->db->queryAllRecords("SELECT * FROM ?? WHERE sys_groupid = ?", $table, $client_group_id);
					$number = count($records);
					if($number > 0) foreach($records as $rec) $xml_data=$this->add_xml($xml_data, $rec, $table);
				}
			}

			//* GDPR
			$gdpr_client=$app->db->queryOneRecord("SELECT * FROM gdpr_client_settings WHERE client_id = ?", $client_id);

			//* Billing
			$invoice_client=$app->db->queryOneRecord("SELECT * FROM invoice_client_settings WHERE client_id = ?", $client_id);
		} else {
			$client=$app->gdpr_functions_data->get_stored_client($client_id);
			$xml_data=$this->add_xml($xml_data, $client, 'client');

			$domains=$app->gdpr_functions_data->get_stored_domains($client_id);
			if(is_array($domains) && !empty($domains)) {
				$out=array('name' => 'domains');
				foreach($domains as $domain) array_push($out, array('name'=>'domain', 'value'=>$domain['domain']));
				array_push($xml_data, $out);
			}

			$gdpr_client=$app->gdpr_functions_data->get_stored_gdpr_client($client_id);
			$invoice_client=$app->gdpr_functions_data->get_stored_invoice_client($client_id);
		}	

		if(is_array($gdpr_client) && !empty($gdpr_client)) {
			$xml_data=$this->add_xml($xml_data, $gdpr_client, 'gdpr');
			$gdprs=$app->gdpr_functions_data->get_stored_gdpr($client_id, $client_group_id);
			$out=array('name' => 'gdpr');
			foreach($gdprs as $gdpr=>$val) {
				foreach($val as $i=>$v) array_push($out, array('name' => $i, 'value'=>$v));
			}
			array_push($xml_data, $out);
			if($client_group_id > 1 && $client_id > 0) { //* Full XML
				$tables = 'gdpr_contract';
				$tables_array = explode(',', $tables);
				$table_list = array();
				foreach($tables_array as $table) {
					if($table != '') {
						$records = $app->db->queryAllRecords("SELECT * FROM ?? WHERE client_id = ?", $table, $client_id);
						$number = count($records);
						if($number > 0) {//foreach($records as $rec) $xml_data=$this->add_xml($xml_data, $rec, $table);
							$out=array('name' => $table);
							foreach($records as $rec=>$val) {
								foreach($val as $i=>$v) array_push($out, array('name' => $i, 'value'=>$v));
							}
							array_push($xml_data, $out);
						}
					}
				}
			}
		}

		if(is_array($invoice_client) && !empty($invoice_client)) {
			$xml_data=$this->add_xml($xml_data, $invoice_client, 'billing');
			$invoices=$app->gdpr_functions_data->get_stored_invoice($client_id, $client_group_id);
			$out=array('name' => 'invoices');
			foreach($invoices as $invoice=>$val) {
				foreach($val as $i=>$v) array_push($out, array('name' => $i, 'value'=>$v));
			}
			array_push($xml_data, $out);

			if($client_group_id > 1 && $client_id > 0) { //* Full XML
				$tables = 'invoice_message,invoice_order,invoice_payment_log,invoice_recurring_item,invoice_sepa_mandate';
				$tables_array = explode(',', $tables);
				$table_list = array();
				foreach($tables_array as $table) {
					if($table != '') {
						$records = $app->db->queryAllRecords("SELECT * FROM ?? WHERE client_id = ?", $table, $client_id);
						$number = count($records);
						if($number > 0) {//foreach($records as $rec) $xml_data=$this->add_xml($xml_data, $rec, $table);
							$out=array('name' => $table);
							foreach($records as $rec=>$val) {
								foreach($val as $i=>$v) array_push($out, array('name' => $i, 'value'=>$v));
							}
							array_push($xml_data, $out);
						}
					}
				}
			}
		}

		//* Generate XML

		$doc = new DOMDocument();
		$child = $this->generate_xml_element($doc, $xml_data);
		if ($child) $doc->appendChild($child);
		$doc->formatOutput = true; 
		$xml = $doc->saveXML();
		return $xml;

	}
}
	
?>
