<?php

/*
Copyright (c) 2018, Florian Schaal, schaal @it UG
All rights reserved.
*/

class gdpr_plugin {
	var $plugin_name = 'gdpr_server_plugin';
	var $class_name = 'gdpr_server_plugin';

	function onInstall() {
		return true;
	}

	function onLoad() {
		global $app;
		$app->plugins->registerEvent('gdpr_update', 'gdpr_plugin', 'update');
	}

	function update($event_name, $data) {
		global $app, $conf;

		define('UPDATE_FOLDER', 'ispconfig-gdpr');
		define('UPDATE_ARCHIVE', 'gdpr-module-update.tgz');

		if(!is_file('/tmp/'.UPDATE_ARCHIVE)) return true;

		$error = false;

		$server_id = intval($conf['server_id']);

		if($server_id != 1) {
			$this->log('Updates can not installed on mirror-server', LOGLEVEL_WARN);
		}
		$server_config = $app->getconf->get_server_config($conf['server_id'], 'server');
		$database = $conf['db_database'];
		$backup_dir = trim($server_config['backup_dir']);
		$backup_path = $backup_dir.'/gdpr_'.@date('Y-m-d_H-i');
		$backups="/usr/local/ispconfig/interface/lib/classes/gdpr* /usr/local/ispconfig/interface/lib/plugins/gdpr* /usr/local/ispconfig/interface/gdpr /usr/local/ispconfig/interface/web/gdpr /usr/local/ispconfig/server/mods-available/gdpr* /usr/local/ispconfig/server/plugins-available/gdpr*";
		$backup_tables = "gdpr_action gdpr_client_data gdpr_client_settings gdpr_company gdpr_contract gdpr_settings gdpr_email_templates gdpr_templates";

		if($server_config['backup_dir_is_mount'] == 'y' && !$app->system->mount_backup_dir($backup_dir)) $error = $this->log('Unable to mount the backup-dir. Update aborted', LOGLEVEL_ERROR);
		if(!$error) {
			exec("mkdir -p $backup_path");
			exec("chown root:root $backup_path");
			exec("chmod 700 $backup_path");
			$this->log("Creating backup", LOGLEVEL_DEBUG);
			exec("tar pcfz $backup_path/ispconfig_gdpr.tar.gz $backups 2> /dev/null", $out, $returnvar);
			if($returnvar != 0) $error = $this->log("Backup failed. We stop here.", LOGLEVEL_ERROR);
			if(!$error) {
				include '/usr/local/ispconfig/server/lib/mysql_clientdb.conf';
				exec("mysqldump -u ".escapeshellarg($clientdb_user)." -p".escapeshellarg($clientdb_password)." -c -t --add-drop-table --create-options --quick --result-file=$backup_path/gdpr-".$conf['mysql']['database'].".sql ".escapeshellarg($conf['db_database'])." ".$backup_tables);
				exec("chown root:root $backup_path/*.tar.gz");
				exec("chown root:root $backup_path/*.sql");
				exec("chmod 600 $backup_path/*.tar.gz");
				exec("chmod 600 $backup_path/*.sql");
				chdir('/tmp');
				exec('tar xfz '.UPDATE_ARCHIVE, $ret, $val);
				if($val != 0) $error = $this->log("extracting $current_dir/".UPDATE_ARCHIVE." failed with error $val", LOGLEVEL_ERROR);
				unset($ret);unset($val);
				if(!$error) {
					chdir('/tmp/'.UPDATE_FOLDER);
					include_once '/usr/local/ispconfig/server/lib/config.inc.php';
					// Copy the files
					exec('chown -R ispconfig:ispconfig interface');
					exec('chmod -R 750 interface');
					exec('cp -prf interface /usr/local/ispconfig');
					exec('chmod -R 770 /usr/local/ispconfig/interface/web/gdpr/lib/lang');

					exec('chmod 750 server/scripts/gdpr');
					exec('chmod 700 server/scripts/gdpr/gdpr_update.sh');
					exec('chown root.root server/scripts/*');
					exec('mkdir -p /usr/local/ispconfig/server/scripts/gdpr');
					exec('cp -prf server/* /usr/local/ispconfig/server/');

					if(!file_exists('/usr/local/ispconfig/server/mods-enabled/gdpr_module.inc.php'))
						exec('ln -s /usr/local/ispconfig/server/mods-available/gdpr_module.inc.php /usr/local/ispconfig/server/mods-enabled/gdpr_module.inc.php');
					if(!file_exists('/usr/local/ispconfig/server/plugins-enabled/gdpr_module.inc.php'))
						exec('ln -s /usr/local/ispconfig/server/plugins-available/gdpr_plugin.inc.php /usr/local/ispconfig/server/plugins-enabled/gdpr_module.inc.php');

					$tmp = $app->db->queryOneRecord("SELECT version, rev FROM gdpr_settings WHERE gdpr_settings_id = 1");
					$version = $tmp['version'];
					$current_db_version = $tmp['rev'];
					unset($tmp);
					$force_db_update = true;
					$found = true;
					while($found == true) {
						$next_db_version = intval($current_db_version + 1);
						$patch_filename = 'sql/incremental/upd_'.str_pad($next_db_version, 4, '0', STR_PAD_LEFT).'.sql';
						$php_patch_filename = 'patches/upd_'.str_pad($next_db_version, 4, '0', STR_PAD_LEFT).'.php';
						if(is_file($patch_filename)) {
							echo "RUN PATCH";
							if(is_file($php_patch_filename)) {
								$php_patch_class_name = 'upd_'.str_pad($next_db_version, 4, '0', STR_PAD_LEFT);
								include_once $php_patch_filename;
								if(class_exists($php_patch_class_name)) {
									$php_patch = new $php_patch_class_name;
								}
							}
							if(isset($php_patch) && is_object($php_patch) && method_exists($php_patch, 'onBeforeSQL')) {
								$php_patch->onBeforeSQL();
								echo "Executing PHP patch file $php_patch_filename\n";
							}
							$cmd = "mysql --default-character-set=".escapeshellarg($conf['db_charset'])." --force -h '".escapeshellarg($clientdb_host)."' -u '".escapeshellarg($clientdb_user)."' -p'".escapeshellarg($clientdb_password)."' ".escapeshellarg($conf['db_database'])." < ".$patch_filename;
							if($force_db_update == true) $cmd .= ' > /dev/null 2> /dev/null';
							system($cmd);
							$this->log('Loading SQL patch file'.': '.$patch_filename, LOGLEVEL_DEBUG);

							if(isset($php_patch) && is_object($php_patch) && method_exists($php_patch, 'onAfterSQL')) {
								$php_patch->onAfterSQL();
							}
							$current_db_version = $next_db_version;
						} else {
							$found = false;
						}
					}
					if(is_file('version')) $version = file_get_contents('version');
					$app->db->query("UPDATE gdpr_settings SET rev = ?, version = ?, `update` = 'finished' WHERE gdpr_settings_id = 1", $current_db_version, $version);
					$this->log("ISPConfig GDPR module has been updated", LOGLEVEL_DEBUG);

					chdir('/tmp');
					unlink(UPDATE_ARCHIVE);
					exec('rm -rf '.UPDATE_FOLDER);
					@unlink('/usr/local/ispconfig/interface/web/gdpr/version.cache');
					@unlink('/usr/local/ispconfig/interface/web/gdpr/license.cache');
					return true;

				}
			}
		}
	}

	function log($msg, $level) {
		global $app;
		$log_prefix='GDPR Update';
		$app->log('GDPR Update: '.$msg, $level);
		if($level == LOGLEVEL_ERROR) {
			$app->db->query("UPDATE gdpr_settings SET `update` = 'failed' WHERE gdpr_settings_id = 1");
			return false; 
		}
	}
}
class installer_gdpr_patch_update {
	public function onBeforeSQL() {
		global $app;
	}
	public function onAfterSQL() {
		global $app;
	}
}

?>
